using System;
using System.Collections.Generic;
using System.Linq;
using StoredProcWaApi;
using StoredProcWaApi.BusinessLayer;
using StoredProcWaApi.Models;
// using System.Windows.Forms;    // Note: remove comment when using with windows forms

// for use with web api
using Newtonsoft.Json;
using System.Text;

/// <summary>
/// These are data-centric code examples for the Categories table.
/// You can cut and paste the respective codes into your application
/// by changing the sample values assigned from these examples.
/// NOTE: This class is sealed and contains private methods because they're
/// not meant to be called by an outside client.  Each method contains
/// code for the respective example being shown.
/// These examples shows how to call the Business Layer (or the Middler Layer or Middler Tier) from a client.
/// A client can be a Windows Form, WPF app, Silverlight app, ASP.NET Web Form/MVC/Razor app, a Web API, a Class, and many more.
///
/// Note:  The generated web api endpoints encapsulates calls to the (public) business layer classes, 
/// so you can access the web api or the business layer directly when doing a database operation.
/// Or, you can also use both.
///
/// The examples below also shows how to access the web api using the shorthand Functions.HttpClientGetAsync(...) method.
/// Or, using the traditional "using (var client = new HttpClient())".  You can also use your own way to access the web api.
/// </summary>
public sealed class CategoryExample
{
    // web api encapsulates calls to the business layer, so you don't really have to inject the business layer into your constructor.
    // however, if you want to use both web api and business layer in your code, make sure to inject the business layer objects in the constructor
    private readonly ICategoryBusinessLayer _categoryBusinessLayer;

    public CategoryExample(ICategoryBusinessLayer categoryBusinessLayer)
    {
        _categoryBusinessLayer = categoryBusinessLayer;
    }

    /// <summary>
    /// Shows how to Select all records.  It also shows how to sort, bind, and loop through records.
    /// </summary>
    private async void SelectAllAsync()
    {
        // select all records
        // uncomment this code if you would rather access the business layer instead of the web api
        // List<Category> objCategoriesList = await _categoryBusinessLayer.SelectAll();

        // ** code using the web api instead of the business layer *****************************************************************
        List<Category> objCategoriesList = null;

        using (var client = new HttpClient())
        {
            client.BaseAddress = new Uri(Functions.GetWebApiBaseAddress());
            HttpResponseMessage response = await client.GetAsync("CategoryApi/SelectAllAsync");

            if (response.IsSuccessStatusCode)
            {
                var responseBody = response.Content.ReadAsStringAsync().Result;
                objCategoriesList = JsonConvert.DeserializeObject<List<Category>>(responseBody);
            }
        }
        // ** code using the web api instead of the business layer *****************************************************************

        // Example 1:  you can optionally sort the collection in ascending order by your chosen field 
        objCategoriesList.Sort(CategoryBusinessLayer.ByCategoryName);

        // Example 2:  to sort in descending order, add this line to the Sort code in Example 1 
        objCategoriesList.Reverse();

        // Example 3:  directly bind to a GridView - for ASP.NET Web Forms
        // GridView grid = new GridView();
        // grid.DataSource = objCategoriesList;
        // grid.DataBind();

        // Example 4:  loop through all the Categories
        foreach (Category objCategory in objCategoriesList)
        {
            int categoryID = objCategory.CategoryID;
            string categoryName = objCategory.CategoryName;
            string description = objCategory.Description;
        }
    }

    /// <summary>
    /// Shows how to Select all records sorted by column name in either ascending or descending order.
    /// </summary>
    private async void SelectAllWithSortExpression()
    {
        // select all records sorted by CategoryID in ascending order
        string sortBy = "CategoryID"; // ascending order
        //string sortBy = "CategoryID desc"; // descending order

        // uncomment this code if you would rather access the business layer instead of the web api
        // List<Category> objCategoriesList = await _categoryBusinessLayer.SelectAllAsync(sortBy);

        // ** code using the web api instead of the business layer *****************************************************************
        List<Category> objCategoriesList = null;

        using (var client = new HttpClient())
        {
            client.BaseAddress = new Uri(Functions.GetWebApiBaseAddress());
            HttpResponseMessage response = await client.GetAsync("CategoryApi/SelectAllAsync/" + sortBy + "");

            if (response.IsSuccessStatusCode)
            {
                var responseBody = response.Content.ReadAsStringAsync().Result;
                objCategoriesList = JsonConvert.DeserializeObject<List<Category>>(responseBody);
            }
        }
        // ** code using the web api instead of the business layer *****************************************************************
    }

    /// <summary>
    /// Shows how to Select a record by Primary Key.  It also shows how to retrieve Lazily-loaded related Objects.  Related Objects are assigned for each Foreign Key.
    /// </summary>
    private async void SelectByPrimaryKeyAsync()
    {
        int categoryIDSample = 1;

        // select a record by primary key(s)

        // uncomment this code if you would rather access the business layer instead of the web api
        //Category objCategory = await _categoryBusinessLayer.SelectByPrimaryKeyAsync(categoryIDSample);

        // ** code using the web api instead of the business layer *****************************************************************
        Category objCategory = null;

        using (var client = new HttpClient())
        {
            client.BaseAddress = new Uri(Functions.GetWebApiBaseAddress());
            HttpResponseMessage response = await client.GetAsync("CategoryApi/SelectByPrimaryKey/" + categoryIDSample);

            if (response.IsSuccessStatusCode)
            {
                var responseBody = response.Content.ReadAsStringAsync().Result;
                objCategory = JsonConvert.DeserializeObject<Category>(responseBody);
            }
        }
        // ** code using the web api instead of the business layer *****************************************************************

        if (objCategory != null)
        {
            // if record is found, a record is returned
            int categoryID = objCategory.CategoryID;
            string categoryName = objCategory.CategoryName;
            string description = objCategory.Description;
        }
    }

    /// <summary>
    /// The example below shows how to Select the CategoryID and CategoryName columns for use with a with a Drop Down List, Combo Box, Checked Box List, List View, List Box, etc
    /// </summary>
    private async void SelectCategoryDropDownListDataAsync()
    {
        // uncomment this code if you would rather access the business layer instead of the web api
        // List<Category> objCategoriesList = await _categoryBusinessLayer.SelectCategoryDropDownListDataAsync();

        // ** code using the web api instead of the business layer *****************************************************************
        List<Category> objCategoriesList = null;

        using (var client = new HttpClient())
        {
            client.BaseAddress = new Uri(Functions.GetWebApiBaseAddress());
            HttpResponseMessage response = await client.GetAsync("CategoryApi/SelectCategoryDropDownListDataAsync");

            if (response.IsSuccessStatusCode)
            {
                var responseBody = response.Content.ReadAsStringAsync().Result;
                objCategoriesList = JsonConvert.DeserializeObject<List<Category>>(responseBody);
            }
        }
        // ** code using the web api instead of the business layer *****************************************************************
    }

    /// <summary>
    /// Shows how to Insert or Create a New Record
    /// </summary>
    private async void Insert()
    {
        // first instantiate a new Category
        Category objCategory = new();

        // assign values you want inserted
        objCategory.CategoryName = "Beverages";
        objCategory.Description = "Soft drinks, coffees, teas, beers, and ales";

        // ** code using the web api instead of the business layer *****************************************************************
        // finally, insert a new record
        string serializedModel = JsonConvert.SerializeObject(objCategory);
        
        using (var client = new HttpClient())
        {
            client.BaseAddress = new Uri(Functions.GetWebApiBaseAddress());
            HttpResponseMessage response = await client.PostAsync("CategoryApi/InsertAsync", new StringContent(serializedModel, Encoding.UTF8, "application/json"));

            if (!response.IsSuccessStatusCode)
                throw new Exception("Error Status Code: " + response.StatusCode.ToString() + " Error Reason: " + response.ReasonPhrase + " Error Message: " + response.RequestMessage.ToString());
        }
        // ** code using the web api instead of the business layer *****************************************************************
    }

    /// <summary>
    /// Shows how to Update an existing record by Primary Key
    /// </summary>
    private async void UpdateAsync()
    {
        // first instantiate a new Category
        Category objCategory = new();

        // assign the existing primary key(s)
        // of the record you want updated
        objCategory.CategoryID = 1;

        // assign values you want updated
        objCategory.CategoryName = "Beverages";
        objCategory.Description = "Soft drinks, coffees, teas, beers, and ales";

        // ** code using the web api instead of the business layer *****************************************************************
        string serializedModel = JsonConvert.SerializeObject(objCategory);
        
        using (var client = new HttpClient())
        {
            client.BaseAddress = new Uri(Functions.GetWebApiBaseAddress());
            HttpResponseMessage response = await client.PostAsync("CategoryApi/UpdateAsync", new StringContent(serializedModel, Encoding.UTF8, "application/json"));

            if (!response.IsSuccessStatusCode)
                throw new Exception("Error Status Code: " + response.StatusCode.ToString() + " Error Reason: " + response.ReasonPhrase + " Error Message: " + response.RequestMessage.ToString());
        }
        // ** code using the web api instead of the business layer *****************************************************************
    }

    /// <summary>
    /// Shows how to Delete an existing record by Primary Key
    /// </summary>
    private async void DeleteAsync()
    {
        // delete a record by primary key
        // using business layer;
        //await _categoryBusinessLayer.DeleteAsync(9);

        // ** code using the web api instead of the business layer *****************************************************************
        using (var client = new HttpClient())
        {
            client.BaseAddress = new Uri(Functions.GetWebApiBaseAddress());
            HttpResponseMessage response = await client.DeleteAsync("CategoryApi/DeleteAsync/" + 9);

            if (!response.IsSuccessStatusCode)
                throw new Exception("Error Status Code: " + response.StatusCode.ToString() + " Error Reason: " + response.ReasonPhrase + " Error Message: " + response.RequestMessage.ToString());
        }
        // ** code using the web api instead of the business layer *****************************************************************
    }

    /// <summary>
    /// Shows how to Delete Multiple records by Primary Key
    /// </summary>
    private async void DeleteMultpleAsync()
    {
        // sample data only, in the real world you should use different values
        string ids = "9, 9";

        // split ids into a List
        //List<Int32> categoryIDList = ids.Split(",").Select(Int32.Parse).ToList();

        // using business layer, delete multiple records based on a list of ids (primary keys)
        //await _categoryBusinessLayer.DeleteMultipleAsync(categoryIDList);

        // ** code using the web api instead of the business layer *****************************************************************
        using (var client = new HttpClient())
        {
            client.BaseAddress = new Uri(Functions.GetWebApiBaseAddress());
            HttpResponseMessage response = await client.DeleteAsync("CategoryApi/DeleteMultiple/?ids=" + ids);

            if (!response.IsSuccessStatusCode)
                throw new Exception("Error Status Code: " + response.StatusCode.ToString() + " Error Reason: " + response.ReasonPhrase + " Error Message: " + response.RequestMessage.ToString());
        }
        // ** code using the web api instead of the business layer *****************************************************************
    }

    /// <summary>
    /// Shows how to get the total number of records
    /// </summary>
    private async void GetRecordCountAsync()
    {
        // get the total number of records in the Category table
        int totalRecordCount;

        // using the business layer
        //totalRecordCount = await _categoryBusinessLayer.GetRecordCountAsync();

        // ** code using the web api instead of the business layer *****************************************************************
        using (var client = new HttpClient())
        {
            client.BaseAddress = new Uri(Functions.GetWebApiBaseAddress());
            HttpResponseMessage response = await client.GetAsync("CategoryApi/GetRecordCountAsync/");
        
            if (!response.IsSuccessStatusCode)
            {
                throw new Exception("Error Status Code: " + response.StatusCode.ToString() + " Error Reason: " + response.ReasonPhrase + " Error Message: " + response.RequestMessage.ToString());
            }
            else
            {
                var responseBody = response.Content.ReadAsStringAsync().Result;
                totalRecordCount = JsonConvert.DeserializeObject<int>(responseBody);
            }
        }
        // ** code using the web api instead of the business layer *****************************************************************
    }

    /// <summary>
    /// Shows how to get the total number of records based on Search Parameters.
    /// </summary>
    private async void GetRecordCountDynamicWhereAsync()
    {
        // search parameters, everything is nullable, only items being searched for should be filled
        // note: fields with String type uses a LIKE search, everything else uses an exact match
        // also, every field you're searching for uses the AND operator
        // e.g. int? productID = 1; string productName = "ch";
        // will translate to: SELECT....WHERE productID = 1 AND productName LIKE '%ch%'
        int? categoryID = null;
        string categoryName = null;

        int totalRecordCount = 0;

        // using business layer
        //totalRecordCount = await _categoryBusinessLayer.GetRecordCountDynamicWhereAsync(categoryID, categoryName);

        // using web api
        string responseBody1 = await Functions.HttpClientGetAsync("CategoryApi/GetRecordCountDynamicWhere/?categoryID=" + categoryID + "&categoryName=" + categoryName);
        totalRecordCount = JsonConvert.DeserializeObject<int>(responseBody1);
    }

    /// <summary>
    /// Shows how to get a specific number of sorted records, starting from an index.  The total number of records are also retrieved when using the SelectSkipAndTake() method.
    /// For example, if there are 200 records take only 10 records (numberOfRecordsToRetrieve), starting from the first index (startRetrievalFromRecordIndex = 0)
    /// The example below uses some variables, here are their definitions:
    /// totalRecordCount - total number of records if you were to retrieve everything
    /// startRetrievalFromRecordIndex - the index to start taking records from. Zero (0) E.g. If you want to skip the first 20 records, then assign 19 here.
    /// numberOfRecordsToRetrieve - take n records starting from the startRetrievalFromRecordIndex
    /// sortBy - to sort in Ascending order by Field Name, just assign just the Field Name, do not pass 'asc'
    /// sortBy - to sort in Descending order by Field Name, use the Field Name, a space and the word 'desc'
    /// </summary>
    private async void SelectSkipAndTakeAsync()
    {
        int numberOfRecordsToRetrieve = 10;
        //string sortByDesc = "CategoryID desc";

        // 1. select a specific number of sorted records starting from the index you specify
        List<Category> objCategoriesList;

        // using business layer
        //int startRetrievalFromRecordIndex = 0;
        //string sortByAsc = "CategoryID";
        //objCategoriesList = await _categoryBusinessLayer.SelectSkipAndTakeAsync(numberOfRecordsToRetrieve, startRetrievalFromRecordIndex, sortByAsc);

        // using web api
        int page = 1;
        string sortBy = "CategoryID";
        string sortOrder = "";    // ascending
        //string sortOrder = "desc";    // desscending

        string responseBody2 = await Functions.HttpClientGetAsync("CategoryApi/SelectSkipAndTake/?rows=" + numberOfRecordsToRetrieve + "&page=" + page + "&sidx=" + sortBy + "&sord=" + sortOrder);

        // make sure responseBody2 is not empty before deserialization
        if(!String.IsNullOrEmpty(responseBody2))
            objCategoriesList = JsonConvert.DeserializeObject<List<Category>>(responseBody2);

        // to use objCategoriesList please see the SelectAll() method examples
        // No need for Examples 1 and 2 because the Collection here is already sorted

        // Example 2:  directly bind to a GridView - for ASP.NET Web Forms
        // Example 3:  loop through all the Category(s).  The example above will only loop for 10 items.
    }

    /// <summary>
    /// Shows how to get a specific number of sorted records, starting from an index, based on Search Parameters.  The number of records are also retrieved.
    /// </summary>
    private async void SelectSkipAndTakeDynamicWhereAsync()
    {
        int startRetrievalFromRecordIndex = 0;
        int numberOfRecordsToRetrieve = 10;
        string sortBy = "CategoryID";
        //string sortBy = "CategoryID desc";

        List<Category> objCategoriesList;

        // search parameters, everything is nullable, only items being searched for should be filled
        // note: fields with String type uses a LIKE search, everything else uses an exact match
        // also, every field you're searching for uses the AND operator
        // e.g. int? productID = 1; string productName = "ch";
        // will translate to: SELECT....WHERE productID = 1 AND productName LIKE '%ch%'
        int? categoryID = null;
        string categoryName = null;

        // using business layer
        // 1. select a specific number of sorted records starting from the index you specify based on Search Parameters
        //objCategoriesList = await _categoryBusinessLayer.SelectSkipAndTakeDynamicWhereAsync(categoryID, categoryName, numberOfRecordsToRetrieve, startRetrievalFromRecordIndex, sortBy);

        // using web api
        string responseBody2 = await Functions.HttpClientGetAsync("CategoryApi/SelectSkipAndTakeDynamicWhere/?categoryID=" + categoryID + "&categoryName=" + categoryName + "&rows=" + numberOfRecordsToRetrieve + "&startRowIndex=" + startRetrievalFromRecordIndex + "&sortByExpression=" + sortBy);

        // make sure responseBody2 is not empty before deserialization
        if(!String.IsNullOrEmpty(responseBody2))
            objCategoriesList = JsonConvert.DeserializeObject<List<Category>>(responseBody2);

        // to use objCategoriesList please see the SelectAll() method examples
        // No need for Examples 1 and 2 because the Collection here is already sorted
        // Example 3:  directly bind to a GridView - for ASP.NET Web Forms
        // Example 4:  loop through all the Categories.  The example above will only loop for 10 items.
    }

    /// <summary>
    /// Shows how to get all records based on Search Parameters.
    /// </summary>
    private async void SelectAllDynamicWhereAsync()
    {
        List<Category> objCategoriesList;

        // search parameters, everything is nullable, only items being searched for should be filled
        // note: fields with String type uses a LIKE search, everything else uses an exact match
        // also, every field you're searching for uses the AND operator
        // e.g. int? productID = 1; string productName = "ch";
        // will translate to: SELECT....WHERE productID = 1 AND productName LIKE '%ch%'
        int? categoryID = null;
        string categoryName = null;

        // using business layer
        //objCategoriesList = await _categoryBusinessLayer.SelectAllDynamicWhereAsync(categoryID, categoryName);

        // using web api
        string responseBody2 = await Functions.HttpClientGetAsync("CategoryApi/SelectAllDynamicWhere/?categoryID=" + categoryID + "&categoryName=" + categoryName);

        // make sure responseBody2 is not empty before deserialization
        if(!String.IsNullOrEmpty(responseBody2))
            objCategoriesList = JsonConvert.DeserializeObject<List<Category>>(responseBody2);
    }
}
