using System;
using System.Collections.Generic;
using System.Linq;
using StoredProcWaApi;
using StoredProcWaApi.BusinessLayer;
using StoredProcWaApi.Models;
// using System.Windows.Forms;    // Note: remove comment when using with windows forms

// for use with web api
using Newtonsoft.Json;
using System.Text;

/// <summary>
/// These are data-centric code examples for the Suppliers table.
/// You can cut and paste the respective codes into your application
/// by changing the sample values assigned from these examples.
/// NOTE: This class is sealed and contains private methods because they're
/// not meant to be called by an outside client.  Each method contains
/// code for the respective example being shown.
/// These examples shows how to call the Business Layer (or the Middler Layer or Middler Tier) from a client.
/// A client can be a Windows Form, WPF app, Silverlight app, ASP.NET Web Form/MVC/Razor app, a Web API, a Class, and many more.
///
/// Note:  The generated web api endpoints encapsulates calls to the (public) business layer classes, 
/// so you can access the web api or the business layer directly when doing a database operation.
/// Or, you can also use both.
///
/// The examples below also shows how to access the web api using the shorthand Functions.HttpClientGetAsync(...) method.
/// Or, using the traditional "using (var client = new HttpClient())".  You can also use your own way to access the web api.
/// </summary>
public sealed class SupplierExample
{
    // web api encapsulates calls to the business layer, so you don't really have to inject the business layer into your constructor.
    // however, if you want to use both web api and business layer in your code, make sure to inject the business layer objects in the constructor
    private readonly ISupplierBusinessLayer _supplierBusinessLayer;

    public SupplierExample(ISupplierBusinessLayer supplierBusinessLayer)
    {
        _supplierBusinessLayer = supplierBusinessLayer;
    }

    /// <summary>
    /// Shows how to Select all records.  It also shows how to sort, bind, and loop through records.
    /// </summary>
    private async void SelectAllAsync()
    {
        // select all records
        // uncomment this code if you would rather access the business layer instead of the web api
        // List<Supplier> objSuppliersList = await _supplierBusinessLayer.SelectAll();

        // ** code using the web api instead of the business layer *****************************************************************
        List<Supplier> objSuppliersList = null;

        using (var client = new HttpClient())
        {
            client.BaseAddress = new Uri(Functions.GetWebApiBaseAddress());
            HttpResponseMessage response = await client.GetAsync("SupplierApi/SelectAllAsync");

            if (response.IsSuccessStatusCode)
            {
                var responseBody = response.Content.ReadAsStringAsync().Result;
                objSuppliersList = JsonConvert.DeserializeObject<List<Supplier>>(responseBody);
            }
        }
        // ** code using the web api instead of the business layer *****************************************************************

        // Example 1:  you can optionally sort the collection in ascending order by your chosen field 
        objSuppliersList.Sort(SupplierBusinessLayer.ByCompanyName);

        // Example 2:  to sort in descending order, add this line to the Sort code in Example 1 
        objSuppliersList.Reverse();

        // Example 3:  directly bind to a GridView - for ASP.NET Web Forms
        // GridView grid = new GridView();
        // grid.DataSource = objSuppliersList;
        // grid.DataBind();

        // Example 4:  loop through all the Suppliers
        foreach (Supplier objSupplier in objSuppliersList)
        {
            int supplierID = objSupplier.SupplierID;
            string companyName = objSupplier.CompanyName;
            string contactName = objSupplier.ContactName;
            string contactTitle = objSupplier.ContactTitle;
            string address = objSupplier.Address;
            string city = objSupplier.City;
            string region1 = objSupplier.Region1;
            string postalCode = objSupplier.PostalCode;
            string country = objSupplier.Country;
            string phone = objSupplier.Phone;
            string fax = objSupplier.Fax;
            string homePage = objSupplier.HomePage;
        }
    }

    /// <summary>
    /// Shows how to Select all records sorted by column name in either ascending or descending order.
    /// </summary>
    private async void SelectAllWithSortExpression()
    {
        // select all records sorted by SupplierID in ascending order
        string sortBy = "SupplierID"; // ascending order
        //string sortBy = "SupplierID desc"; // descending order

        // uncomment this code if you would rather access the business layer instead of the web api
        // List<Supplier> objSuppliersList = await _supplierBusinessLayer.SelectAllAsync(sortBy);

        // ** code using the web api instead of the business layer *****************************************************************
        List<Supplier> objSuppliersList = null;

        using (var client = new HttpClient())
        {
            client.BaseAddress = new Uri(Functions.GetWebApiBaseAddress());
            HttpResponseMessage response = await client.GetAsync("SupplierApi/SelectAllAsync/" + sortBy + "");

            if (response.IsSuccessStatusCode)
            {
                var responseBody = response.Content.ReadAsStringAsync().Result;
                objSuppliersList = JsonConvert.DeserializeObject<List<Supplier>>(responseBody);
            }
        }
        // ** code using the web api instead of the business layer *****************************************************************
    }

    /// <summary>
    /// Shows how to Select a record by Primary Key.  It also shows how to retrieve Lazily-loaded related Objects.  Related Objects are assigned for each Foreign Key.
    /// </summary>
    private async void SelectByPrimaryKeyAsync()
    {
        int supplierIDSample = 1;

        // select a record by primary key(s)

        // uncomment this code if you would rather access the business layer instead of the web api
        //Supplier objSupplier = await _supplierBusinessLayer.SelectByPrimaryKeyAsync(supplierIDSample);

        // ** code using the web api instead of the business layer *****************************************************************
        Supplier objSupplier = null;

        using (var client = new HttpClient())
        {
            client.BaseAddress = new Uri(Functions.GetWebApiBaseAddress());
            HttpResponseMessage response = await client.GetAsync("SupplierApi/SelectByPrimaryKey/" + supplierIDSample);

            if (response.IsSuccessStatusCode)
            {
                var responseBody = response.Content.ReadAsStringAsync().Result;
                objSupplier = JsonConvert.DeserializeObject<Supplier>(responseBody);
            }
        }
        // ** code using the web api instead of the business layer *****************************************************************

        if (objSupplier != null)
        {
            // if record is found, a record is returned
            int supplierID = objSupplier.SupplierID;
            string companyName = objSupplier.CompanyName;
            string contactName = objSupplier.ContactName;
            string contactTitle = objSupplier.ContactTitle;
            string address = objSupplier.Address;
            string city = objSupplier.City;
            string region1 = objSupplier.Region1;
            string postalCode = objSupplier.PostalCode;
            string country = objSupplier.Country;
            string phone = objSupplier.Phone;
            string fax = objSupplier.Fax;
            string homePage = objSupplier.HomePage;
        }
    }

    /// <summary>
    /// The example below shows how to Select the SupplierID and CompanyName columns for use with a with a Drop Down List, Combo Box, Checked Box List, List View, List Box, etc
    /// </summary>
    private async void SelectSupplierDropDownListDataAsync()
    {
        // uncomment this code if you would rather access the business layer instead of the web api
        // List<Supplier> objSuppliersList = await _supplierBusinessLayer.SelectSupplierDropDownListDataAsync();

        // ** code using the web api instead of the business layer *****************************************************************
        List<Supplier> objSuppliersList = null;

        using (var client = new HttpClient())
        {
            client.BaseAddress = new Uri(Functions.GetWebApiBaseAddress());
            HttpResponseMessage response = await client.GetAsync("SupplierApi/SelectSupplierDropDownListDataAsync");

            if (response.IsSuccessStatusCode)
            {
                var responseBody = response.Content.ReadAsStringAsync().Result;
                objSuppliersList = JsonConvert.DeserializeObject<List<Supplier>>(responseBody);
            }
        }
        // ** code using the web api instead of the business layer *****************************************************************
    }

    /// <summary>
    /// Shows how to Insert or Create a New Record
    /// </summary>
    private async void Insert()
    {
        // first instantiate a new Supplier
        Supplier objSupplier = new();

        // assign values you want inserted
        objSupplier.CompanyName = "Exotic Liquids";
        objSupplier.ContactName = "Charlotte Cooper";
        objSupplier.ContactTitle = "Purchasing Manager";
        objSupplier.Address = "49 Gilbert St.";
        objSupplier.City = "London";
        objSupplier.Region1 = "abc";
        objSupplier.PostalCode = "EC1 4SD";
        objSupplier.Country = "UK";
        objSupplier.Phone = "(171) 555-2222";
        objSupplier.Fax = "abc";
        objSupplier.HomePage = "abc";

        // ** code using the web api instead of the business layer *****************************************************************
        // finally, insert a new record
        string serializedModel = JsonConvert.SerializeObject(objSupplier);
        
        using (var client = new HttpClient())
        {
            client.BaseAddress = new Uri(Functions.GetWebApiBaseAddress());
            HttpResponseMessage response = await client.PostAsync("SupplierApi/InsertAsync", new StringContent(serializedModel, Encoding.UTF8, "application/json"));

            if (!response.IsSuccessStatusCode)
                throw new Exception("Error Status Code: " + response.StatusCode.ToString() + " Error Reason: " + response.ReasonPhrase + " Error Message: " + response.RequestMessage.ToString());
        }
        // ** code using the web api instead of the business layer *****************************************************************
    }

    /// <summary>
    /// Shows how to Update an existing record by Primary Key
    /// </summary>
    private async void UpdateAsync()
    {
        // first instantiate a new Supplier
        Supplier objSupplier = new();

        // assign the existing primary key(s)
        // of the record you want updated
        objSupplier.SupplierID = 1;

        // assign values you want updated
        objSupplier.CompanyName = "Exotic Liquids";
        objSupplier.ContactName = "Charlotte Cooper";
        objSupplier.ContactTitle = "Purchasing Manager";
        objSupplier.Address = "49 Gilbert St.";
        objSupplier.City = "London";
        objSupplier.Region1 = "abc";
        objSupplier.PostalCode = "EC1 4SD";
        objSupplier.Country = "UK";
        objSupplier.Phone = "(171) 555-2222";
        objSupplier.Fax = "abc";
        objSupplier.HomePage = "abc";

        // ** code using the web api instead of the business layer *****************************************************************
        string serializedModel = JsonConvert.SerializeObject(objSupplier);
        
        using (var client = new HttpClient())
        {
            client.BaseAddress = new Uri(Functions.GetWebApiBaseAddress());
            HttpResponseMessage response = await client.PostAsync("SupplierApi/UpdateAsync", new StringContent(serializedModel, Encoding.UTF8, "application/json"));

            if (!response.IsSuccessStatusCode)
                throw new Exception("Error Status Code: " + response.StatusCode.ToString() + " Error Reason: " + response.ReasonPhrase + " Error Message: " + response.RequestMessage.ToString());
        }
        // ** code using the web api instead of the business layer *****************************************************************
    }

    /// <summary>
    /// Shows how to Delete an existing record by Primary Key
    /// </summary>
    private async void DeleteAsync()
    {
        // delete a record by primary key
        // using business layer;
        //await _supplierBusinessLayer.DeleteAsync(30);

        // ** code using the web api instead of the business layer *****************************************************************
        using (var client = new HttpClient())
        {
            client.BaseAddress = new Uri(Functions.GetWebApiBaseAddress());
            HttpResponseMessage response = await client.DeleteAsync("SupplierApi/DeleteAsync/" + 30);

            if (!response.IsSuccessStatusCode)
                throw new Exception("Error Status Code: " + response.StatusCode.ToString() + " Error Reason: " + response.ReasonPhrase + " Error Message: " + response.RequestMessage.ToString());
        }
        // ** code using the web api instead of the business layer *****************************************************************
    }

    /// <summary>
    /// Shows how to Delete Multiple records by Primary Key
    /// </summary>
    private async void DeleteMultpleAsync()
    {
        // sample data only, in the real world you should use different values
        string ids = "30, 30";

        // split ids into a List
        //List<Int32> supplierIDList = ids.Split(",").Select(Int32.Parse).ToList();

        // using business layer, delete multiple records based on a list of ids (primary keys)
        //await _supplierBusinessLayer.DeleteMultipleAsync(supplierIDList);

        // ** code using the web api instead of the business layer *****************************************************************
        using (var client = new HttpClient())
        {
            client.BaseAddress = new Uri(Functions.GetWebApiBaseAddress());
            HttpResponseMessage response = await client.DeleteAsync("SupplierApi/DeleteMultiple/?ids=" + ids);

            if (!response.IsSuccessStatusCode)
                throw new Exception("Error Status Code: " + response.StatusCode.ToString() + " Error Reason: " + response.ReasonPhrase + " Error Message: " + response.RequestMessage.ToString());
        }
        // ** code using the web api instead of the business layer *****************************************************************
    }

    /// <summary>
    /// Shows how to get the total number of records
    /// </summary>
    private async void GetRecordCountAsync()
    {
        // get the total number of records in the Supplier table
        int totalRecordCount;

        // using the business layer
        //totalRecordCount = await _supplierBusinessLayer.GetRecordCountAsync();

        // ** code using the web api instead of the business layer *****************************************************************
        using (var client = new HttpClient())
        {
            client.BaseAddress = new Uri(Functions.GetWebApiBaseAddress());
            HttpResponseMessage response = await client.GetAsync("SupplierApi/GetRecordCountAsync/");
        
            if (!response.IsSuccessStatusCode)
            {
                throw new Exception("Error Status Code: " + response.StatusCode.ToString() + " Error Reason: " + response.ReasonPhrase + " Error Message: " + response.RequestMessage.ToString());
            }
            else
            {
                var responseBody = response.Content.ReadAsStringAsync().Result;
                totalRecordCount = JsonConvert.DeserializeObject<int>(responseBody);
            }
        }
        // ** code using the web api instead of the business layer *****************************************************************
    }

    /// <summary>
    /// Shows how to get the total number of records based on Search Parameters.
    /// </summary>
    private async void GetRecordCountDynamicWhereAsync()
    {
        // search parameters, everything is nullable, only items being searched for should be filled
        // note: fields with String type uses a LIKE search, everything else uses an exact match
        // also, every field you're searching for uses the AND operator
        // e.g. int? productID = 1; string productName = "ch";
        // will translate to: SELECT....WHERE productID = 1 AND productName LIKE '%ch%'
        int? supplierID = null;
        string companyName = null;
        string contactName = null;
        string contactTitle = null;
        string address = null;
        string city = null;
        string region1 = null;
        string postalCode = null;
        string country = null;
        string phone = null;
        string fax = null;

        int totalRecordCount = 0;

        // using business layer
        //totalRecordCount = await _supplierBusinessLayer.GetRecordCountDynamicWhereAsync(supplierID, companyName, contactName, contactTitle, address, city, region1, postalCode, country, phone, fax);

        // using web api
        string responseBody1 = await Functions.HttpClientGetAsync("SupplierApi/GetRecordCountDynamicWhere/?supplierID=" + supplierID + "&companyName=" + companyName + "&contactName=" + contactName + "&contactTitle=" + contactTitle + "&address=" + address + "&city=" + city + "&region1=" + region1 + "&postalCode=" + postalCode + "&country=" + country + "&phone=" + phone + "&fax=" + fax);
        totalRecordCount = JsonConvert.DeserializeObject<int>(responseBody1);
    }

    /// <summary>
    /// Shows how to get a specific number of sorted records, starting from an index.  The total number of records are also retrieved when using the SelectSkipAndTake() method.
    /// For example, if there are 200 records take only 10 records (numberOfRecordsToRetrieve), starting from the first index (startRetrievalFromRecordIndex = 0)
    /// The example below uses some variables, here are their definitions:
    /// totalRecordCount - total number of records if you were to retrieve everything
    /// startRetrievalFromRecordIndex - the index to start taking records from. Zero (0) E.g. If you want to skip the first 20 records, then assign 19 here.
    /// numberOfRecordsToRetrieve - take n records starting from the startRetrievalFromRecordIndex
    /// sortBy - to sort in Ascending order by Field Name, just assign just the Field Name, do not pass 'asc'
    /// sortBy - to sort in Descending order by Field Name, use the Field Name, a space and the word 'desc'
    /// </summary>
    private async void SelectSkipAndTakeAsync()
    {
        int numberOfRecordsToRetrieve = 10;
        //string sortByDesc = "SupplierID desc";

        // 1. select a specific number of sorted records starting from the index you specify
        List<Supplier> objSuppliersList;

        // using business layer
        //int startRetrievalFromRecordIndex = 0;
        //string sortByAsc = "SupplierID";
        //objSuppliersList = await _supplierBusinessLayer.SelectSkipAndTakeAsync(numberOfRecordsToRetrieve, startRetrievalFromRecordIndex, sortByAsc);

        // using web api
        int page = 1;
        string sortBy = "SupplierID";
        string sortOrder = "";    // ascending
        //string sortOrder = "desc";    // desscending

        string responseBody2 = await Functions.HttpClientGetAsync("SupplierApi/SelectSkipAndTake/?rows=" + numberOfRecordsToRetrieve + "&page=" + page + "&sidx=" + sortBy + "&sord=" + sortOrder);

        // make sure responseBody2 is not empty before deserialization
        if(!String.IsNullOrEmpty(responseBody2))
            objSuppliersList = JsonConvert.DeserializeObject<List<Supplier>>(responseBody2);

        // to use objSuppliersList please see the SelectAll() method examples
        // No need for Examples 1 and 2 because the Collection here is already sorted

        // Example 2:  directly bind to a GridView - for ASP.NET Web Forms
        // Example 3:  loop through all the Supplier(s).  The example above will only loop for 10 items.
    }

    /// <summary>
    /// Shows how to get a specific number of sorted records, starting from an index, based on Search Parameters.  The number of records are also retrieved.
    /// </summary>
    private async void SelectSkipAndTakeDynamicWhereAsync()
    {
        int startRetrievalFromRecordIndex = 0;
        int numberOfRecordsToRetrieve = 10;
        string sortBy = "SupplierID";
        //string sortBy = "SupplierID desc";

        List<Supplier> objSuppliersList;

        // search parameters, everything is nullable, only items being searched for should be filled
        // note: fields with String type uses a LIKE search, everything else uses an exact match
        // also, every field you're searching for uses the AND operator
        // e.g. int? productID = 1; string productName = "ch";
        // will translate to: SELECT....WHERE productID = 1 AND productName LIKE '%ch%'
        int? supplierID = null;
        string companyName = null;
        string contactName = null;
        string contactTitle = null;
        string address = null;
        string city = null;
        string region1 = null;
        string postalCode = null;
        string country = null;
        string phone = null;
        string fax = null;

        // using business layer
        // 1. select a specific number of sorted records starting from the index you specify based on Search Parameters
        //objSuppliersList = await _supplierBusinessLayer.SelectSkipAndTakeDynamicWhereAsync(supplierID, companyName, contactName, contactTitle, address, city, region1, postalCode, country, phone, fax, numberOfRecordsToRetrieve, startRetrievalFromRecordIndex, sortBy);

        // using web api
        string responseBody2 = await Functions.HttpClientGetAsync("SupplierApi/SelectSkipAndTakeDynamicWhere/?supplierID=" + supplierID + "&companyName=" + companyName + "&contactName=" + contactName + "&contactTitle=" + contactTitle + "&address=" + address + "&city=" + city + "&region1=" + region1 + "&postalCode=" + postalCode + "&country=" + country + "&phone=" + phone + "&fax=" + fax + "&rows=" + numberOfRecordsToRetrieve + "&startRowIndex=" + startRetrievalFromRecordIndex + "&sortByExpression=" + sortBy);

        // make sure responseBody2 is not empty before deserialization
        if(!String.IsNullOrEmpty(responseBody2))
            objSuppliersList = JsonConvert.DeserializeObject<List<Supplier>>(responseBody2);

        // to use objSuppliersList please see the SelectAll() method examples
        // No need for Examples 1 and 2 because the Collection here is already sorted
        // Example 3:  directly bind to a GridView - for ASP.NET Web Forms
        // Example 4:  loop through all the Suppliers.  The example above will only loop for 10 items.
    }

    /// <summary>
    /// Shows how to get all records based on Search Parameters.
    /// </summary>
    private async void SelectAllDynamicWhereAsync()
    {
        List<Supplier> objSuppliersList;

        // search parameters, everything is nullable, only items being searched for should be filled
        // note: fields with String type uses a LIKE search, everything else uses an exact match
        // also, every field you're searching for uses the AND operator
        // e.g. int? productID = 1; string productName = "ch";
        // will translate to: SELECT....WHERE productID = 1 AND productName LIKE '%ch%'
        int? supplierID = null;
        string companyName = null;
        string contactName = null;
        string contactTitle = null;
        string address = null;
        string city = null;
        string region1 = null;
        string postalCode = null;
        string country = null;
        string phone = null;
        string fax = null;

        // using business layer
        //objSuppliersList = await _supplierBusinessLayer.SelectAllDynamicWhereAsync(supplierID, companyName, contactName, contactTitle, address, city, region1, postalCode, country, phone, fax);

        // using web api
        string responseBody2 = await Functions.HttpClientGetAsync("SupplierApi/SelectAllDynamicWhere/?supplierID=" + supplierID + "&companyName=" + companyName + "&contactName=" + contactName + "&contactTitle=" + contactTitle + "&address=" + address + "&city=" + city + "&region1=" + region1 + "&postalCode=" + postalCode + "&country=" + country + "&phone=" + phone + "&fax=" + fax);

        // make sure responseBody2 is not empty before deserialization
        if(!String.IsNullOrEmpty(responseBody2))
            objSuppliersList = JsonConvert.DeserializeObject<List<Supplier>>(responseBody2);
    }
}
