using System;
using System.Collections.Generic;
using System.Data;
using System.Data.SqlClient;
using System.Threading.Tasks;

namespace StoredProcWaApi.DataRepository.Helper
{
    /// <summary>
    /// Helper or Utility methods for use with a database.
    /// </summary>
    internal static class DatabaseFunctions
    {
        /// <summary>
        /// This method will return a new MsSqlDatabase, assigning values to all
        /// MsSqlDatabase properties such as SqlConnection, SqlCommand, and SqlDataAdapter.
        /// </summary>
        /// <param name="connectionString">Connection string</param>
        /// <param name="sql">Stored Procedure name or Ad-Hoc SQL Script</param>
        /// <param name="sqlParamList">List of SqlParameters</param>
        /// <param name="commandType">SqlCommand Type.  Acceptable types are CommandType.StoredProcedure or CommandType.Text only.</param>
        /// <returns>MsSqlDatabase</returns>
        private static MsSqlDatabase GetMsSqlDatabase(string connectionString, string sql, List<SqlParameter> sqlParamList, CommandType commandType)
        {
            // make sure command type is valid
            if (commandType == CommandType.TableDirect)
                throw new ArgumentException("Invalid CommandType!  Acceptable types are CommandType.StoredProcedure or CommandType.Text only.");

            // instantiate sqlConnection
            SqlConnection sqlConnection = new(connectionString);

            // instantiate sqlCommand and assign SqlCommand properties
            SqlCommand sqlCommand = new(sql, sqlConnection);
            sqlCommand.CommandType = commandType;

            foreach (var param in sqlParamList)
            {
                sqlCommand.Parameters.AddWithValue(param.ParameterName, param.Value);
            }

            // instatntiate sqlDataAdapter
            SqlDataAdapter sqlDataAdapter = new(sqlCommand);

            // instantiate msSqlDatabase and return it
            MsSqlDatabase msSqlDatabase = new(sqlConnection, sqlCommand, sqlDataAdapter);
            return msSqlDatabase;
        }

        /// <summary>
        /// This method will return a DataTable.  It will open and close a connection to a MS SQL Database, fill and return the DataTable.
        /// </summary>
        /// <param name="connectionString">Connection string</param>
        /// <param name="sql">Stored Procedure name or Ad-Hoc SQL Script</param>
        /// <param name="sqlParamList">List of SqlParameters</param>
        /// <param name="commandType">SqlCommand Type.  Acceptable types are CommandType.StoredProcedure or CommandType.Text only.</param>
        /// <returns>DataTable</returns>
        internal static async Task<DataTable> GetDataTableAsync(string connectionString, string sql, List<SqlParameter> sqlParamList, CommandType commandType)
        {
            // make sure command type is valid
            if (commandType == CommandType.TableDirect)
                throw new ArgumentException("Invalid CommandType!  Acceptable types are CommandType.StoredProcedure or CommandType.Text only.");

            DataTable dt;
            MsSqlDatabase msSqlDatabase = GetMsSqlDatabase(connectionString, sql, sqlParamList, commandType);

            using (msSqlDatabase.SqlConnection)
            {
                // open SqlConnection to the database
                await msSqlDatabase.SqlConnection.OpenAsync();

                // execute transact-sql command against the database
                using (msSqlDatabase.SqlCommand)
                {
                    // fill the datatable using SqlDataAdapter
                    using (msSqlDatabase.SqlDataAdapter)
                    {
                        dt = new DataTable();
                        msSqlDatabase.SqlDataAdapter.Fill(dt);
                    }
                }
            }

            return dt;
        }

        /// <summary>
        /// This method will ExecuteNonQueryAsync or ExecuteScalarAsync.  Pass a sql script that will:
        /// Create a new record, Update an existing record, or Delete an existing record.
        /// When creating a new record, the newly created id is returned if the id of the database table is automatically generated.
        /// </summary>
        /// <param name="connectionString">Connection string</param>
        /// <param name="sql">Stored Procedure name or Ad-Hoc SQL Script</param>
        /// <param name="sqlParamList">List of SqlParameters</param>
        /// <param name="commandType">SqlCommand Type.  Acceptable types are CommandType.StoredProcedure or CommandType.Text only.</param>
        /// <param name="operationType">Acceptable operation types are Create, Update, or Delete only</param>
        /// <param name="isPrimaryKeyGuid">Optional.  Required when operationType is DatabaseOperationType.Create, must be True or False.</param>
        /// <returns>object</returns>
        internal static async Task<object> ExecuteSqlCommandAsync(string connectionString, string sql, List<SqlParameter> sqlParamList, CommandType commandType, DatabaseOperationType operationType, bool? isPrimaryKeyGuid = null)
        {
            // make sure command type is valid
            if (commandType == CommandType.TableDirect)
                throw new ArgumentException("Invalid CommandType!  Acceptable types are CommandType.StoredProcedure or CommandType.Text only.");

            if (operationType == DatabaseOperationType.RetrieveDataTable)
                throw new ArgumentException("Invalid DatabaseOperationType!  Acceptable operation types are: Create, Update, and Delete only.", "operationType: " + operationType.ToString());

            if (operationType == DatabaseOperationType.Create && isPrimaryKeyGuid == null)
                throw new ArgumentException("Invalid isPrimaryKeyGuid! isPrimaryKeyGuid cannot be null when the operation type is DatabaseOperationType.Create, must be True or False.", "isPrimaryKeyGuid is null.");

            MsSqlDatabase msSqlDatabase = GetMsSqlDatabase(connectionString, sql, sqlParamList, commandType);

            using (msSqlDatabase.SqlConnection)
            {
                // open SqlConnection to the database
                await msSqlDatabase.SqlConnection.OpenAsync();

                // execute transact-sql command against the database
                using (msSqlDatabase.SqlCommand)
                {
                    if (operationType == DatabaseOperationType.Update || operationType == DatabaseOperationType.Delete)
                    {
                        // update or delete record
                        await msSqlDatabase.SqlCommand.ExecuteNonQueryAsync();
                    }
                    else
                    {
                        // create new record
                        if (isPrimaryKeyGuid.Value)
                            return msSqlDatabase.SqlCommand.ExecuteScalar().ToString();

                        return await msSqlDatabase.SqlCommand.ExecuteScalarAsync();
                    }
                }
            }

            return null;
        }

        /// <summary>
        /// Adds start, numberOfRows, and sortByExpress SQL parameters to the sqlParams List.
        /// </summary>
        /// <param name="sqlParamList">List of SqlParameters</param>
        /// <param name="sortByExpression">Sort by Column Name.  E.g. "ProductID" sorts by ProductID in ascending order.  "ProductID desc" sorts by ProductID in descending order.</param>
        /// <param name="startRowIndex">Optional.  Index of the row to start taking data from.  E.g. 0 will start taking data from the 1st row, 9 will start taking data from the 10th row.</param>
        /// <param name="rows">Optional.  Number of rows to take.</param>
        internal static void AddSelectSkipAndTakeParams(List<SqlParameter> sqlParamList, string sortByExpression, int startRowIndex, int rows)
        {
            AddSqlParameter(sqlParamList, "@start", startRowIndex);
            AddSqlParameter(sqlParamList, "@numberOfRows", rows);
            AddSqlParameter(sqlParamList, "@sortByExpression", sortByExpression);
        }
 
        /// <summary>
        /// Adds a SqlParameter to the sqlParams List.
        /// </summary>
        /// <param name="sqlParamList">List of SqlParameters</param>
        /// <param name="parameterName">Parameter Name used to create the SqlParameter.  E.g. "@myParamName" or "@age"</param>
        /// <param name="parameterValue">Parameter Value used to create the SqlParameter.  E.g. "myParamValue" or 30</param>
        internal static void AddSqlParameter(List<SqlParameter> sqlParamList, string parameterName, object parameterValue)
        {
            if (!String.IsNullOrEmpty(parameterName))
            {
                SqlParameter sqlParam = new(parameterName, parameterValue);
                sqlParamList.Add(sqlParam);
            }
        }

        /// <summary>
        /// Gets the Number of Records from the specific table based on the SQL script (Stored Procedure).
        /// </summary>
        /// <param name="connectionString">Connection string</param>
        /// <param name="sql">SQL script</param>
        /// <param name="sqlParamList">List of SqlParameters</param>
        /// <param name="commandType">SqlCommand Type.  Acceptable types are CommandType.StoredProcedure or CommandType.Text only.</param>
        /// <returns>Record Count - int</returns>
        internal static async Task<int> GetRecordCountAsync(string connectionString, string sql, List<SqlParameter> sqlParamList, CommandType commandType)
        {
            int recordCount = 0;

            // get a datatable based on the sql script
            DataTable dt = await GetDataTableAsync(connectionString, sql, sqlParamList, commandType);

            // convert data to an int - record count
            if (dt != null && dt.Rows.Count > 0)
                recordCount = (int)dt.Rows[0]["RecordCount"];

            return recordCount;
        }
    }
}
