using System;
using System.Net.Http;
using System.Text;
using System.Text.RegularExpressions;
using System.Threading.Tasks;
using Microsoft.Extensions.DependencyInjection;
using StoredProcWaApi.Domain;
using StoredProcWaApi.BusinessLayer;
using StoredProcWaApi.DataRepository;
using StoredProcWaApi.Models;
using StoredProcWaApi.ViewModels;
using System.Data.SqlClient;

namespace StoredProcWaApi
{
    /// <summary>
    /// Helper functions or methods used by the current application.
    /// ************************************* Do not make changes to this class *************************************
    /// </summary>
    public sealed class Functions
    {
        private Functions()
        {
        }

        /// <summary>
        /// Removes non-alphanumeric characters other than a Space and Dash.
        /// </summary>
        /// <param name="text">String text that needs to be filtered</param>
        /// <returns>Returns a String.  E.g. 12Abc#$ ef-g will return 12Abc ef-g</returns>
        private static string RemoveSpecialChars(string text)
        {
            Regex regex = new Regex("[^a-zA-Z0-9 -]");
            return regex.Replace(text, "");
        }

        /// <summary>
        /// Returns values used in the SQL's Where statement when searching for a value.
        /// </summary>
        /// <param name="fieldName">Table's Column Name</param>
        /// <param name="data">Value being searched for or filtered</param>
        /// <param name="fieldType">String, Boolean, Numeric, Decimal</param>
        /// <returns>When searching for a String fieldType: E.g.  [MyColumnName] LIKE '%" + data + "%'"</returns>
        public static string GetWhereValue(string fieldName, string data, FieldType fieldType)
        {
            switch (fieldType)
            {
                case FieldType.String:
                    return "[" + fieldName + "] LIKE '%" + data + "%'";
                case FieldType.Date:
                    return "[" + fieldName + "] = '" + data + "'";
                case FieldType.Boolean:
                    if (data == "false")
                        return "([" + fieldName + "] = 0 OR [" + fieldName + "] IS NULL)";
                    else
                        return "[" + fieldName + "] = 1";
                case FieldType.Numeric:
                    if (data == "0")
                        return "([" + fieldName + "] = " + data + " OR [" + fieldName + "] IS NULL)";
                    else
                        return "[" + fieldName + "] = " + data;
                case FieldType.Decimal:
                    if (data == "0" || data == "0.0" || data == "0.00")
                        return "([" + fieldName + "] = " + data + " OR [" + fieldName + "] IS NULL)";
                    else
                        return "[" + fieldName + "] = " + data;
                default:
                    return "[" + fieldName + "] = '" + data + "'";
            }
        }

        /// <summary>
        /// Used in the ListForeach razor view.
        /// Gets the Start Page shown in the footer (pager) of the ListForeach grid along with page numbers.
        /// </summary>
        /// <param name="currentPage">Page number where the current grid is</param>
        /// <param name="numberOfPagesToShow">Number of pages to show in the pager between the First and Last links</param>
        /// <returns>Start Page in the pager</returns>
        public static int GetPagerStartPage(int currentPage, int numberOfPagesToShow)
        {
            int startPage = 1;

            if (currentPage <= numberOfPagesToShow)
                startPage = 1;
            else if ((currentPage > numberOfPagesToShow) && (currentPage % numberOfPagesToShow == 0))
                startPage = ((currentPage / numberOfPagesToShow) - 1) * numberOfPagesToShow + 1;
            else if (currentPage > numberOfPagesToShow)
                startPage = (currentPage / numberOfPagesToShow) * numberOfPagesToShow + 1;

            return startPage;
        }

        /// <summary>
        /// Used in the ListForeach razor view.
        /// Gets the End Page shown in the footer (pager) of the ListForeach grid along with page numbers.
        /// </summary>
        /// <param name="startPage">Page number where the current grid is</param>
        /// <param name="numberOfPagesToShow">Number of pages to show in the pager between the First and Last links</param>
        /// <param name="totalPages">Total number of pages</param>
        /// <returns>Start Page in the pager</returns>
        public static int GetPagerEndPage(int startPage, int numberOfPagesToShow, int totalPages)
        {
            int endPage = startPage + (numberOfPagesToShow - 1);

            if (endPage >= totalPages)
                return totalPages;
            else
                return startPage + (numberOfPagesToShow - 1);
        }

        /// <summary>
        /// The default Base Address for the Web API project.
        /// </summary>
        /// <returns>E.g. "https://localhost:7111/"</returns>
        public static string GetWebApiBaseAddress()
        {
            return "https://localhost:7111/";
        }

        /// <summary>
        /// Used in the ListForeach razor view pager shown in the footer of the grid.
        /// Gets the number of rows to show per page.
        /// </summary>
        public static int GetGridNumberOfRows()
        {
            return 15;
        }

        /// <summary>
        /// Used in the ListForeach razor view pager shown in the footer of the grid.
        /// Gets the number of pages to show in between the First and Last link.
        /// </summary>
        public static int GetGridNumberOfPagesToShow()
        {
            return 10;
        }

        /// <summary>
        /// Calls a Web API URL using HttpClient GetAsync.
        /// </summary>
        /// <param name="webApiUrl">Web API URL</param>
        /// <returns>Returns a String HttpResponseMessage result</returns>
        public static async Task<string> HttpClientGetAsync(string webApiUrl)
        {
            string responseBody = null;

            using (var client = new HttpClient())
            {
                client.BaseAddress = new Uri(Functions.GetWebApiBaseAddress());
                HttpResponseMessage response = await client.GetAsync(webApiUrl);

                if (!response.IsSuccessStatusCode)
                    throw new Exception("Error Status Code: " + response.StatusCode.ToString() + " Error Reason: " + response.ReasonPhrase + " Error Message: " + response.RequestMessage.ToString());
                else
                    responseBody = response.Content.ReadAsStringAsync().Result;
            }

            return responseBody;
        }

        /// <summary>
        /// Calls a Web API URL using HttpClient PostAsync.
        /// </summary>
        /// <param name="webApiUrl">Web API URL</param>
        /// <param name="serializedModel">Serialized json data that will be passed via the PostAsync call</param>
        /// <returns>Returns HttpResponseMessage</returns>
        public static async Task<HttpResponseMessage> HttpClientPostAsync(string webApiUrl, string serializedModel)
        {
            HttpResponseMessage response = null;

            using (var client = new HttpClient())
            {
                client.BaseAddress = new Uri(Functions.GetWebApiBaseAddress());
                response = await client.PostAsync(webApiUrl, new StringContent(serializedModel, Encoding.UTF8, "application/json"));

                if (!response.IsSuccessStatusCode)
                    throw new Exception("Error Status Code: " + response.StatusCode.ToString() + " Error Reason: " + response.ReasonPhrase + " Error Message: " + response.RequestMessage.ToString());
            }

            return response;
        }

        /// <summary>
        /// Calls a Web API URL using HttpClient DeleteAsync.
        /// </summary>
        /// <param name="webApiUrl">Web API URL</param>
        /// <returns>Returns HttpResponseMessage</returns>
        public static async Task<HttpResponseMessage> HttpClientDeleteAsync(string webApiUrl)
        {
            HttpResponseMessage response = null;

            using (var client = new HttpClient())
            {
                client.BaseAddress = new Uri(Functions.GetWebApiBaseAddress());
                response = await client.DeleteAsync(webApiUrl);

                if (!response.IsSuccessStatusCode)
                    throw new Exception("Error Status Code: " + response.StatusCode.ToString() + " Error Reason: " + response.ReasonPhrase + " Error Message: " + response.RequestMessage.ToString());
            }

            return response;
        }

        /// <summary>
        /// Gets the Index where to start retrieving records from and the Total Pages based on the rows per page.
        /// </summary>
        /// <param name="page">Current page the grid is on</param>
        /// <param name="rows">Number of rows to retrieve</param>
        /// <param name="totalRecords">Total number of records</param>
        /// <returns>startRowIndex (Index of the row to start taking data from) and totalPages (Total number of pages)</returns>
        public static (int startRowIndex, int totalPages) GetStartRowIndexAndTotalPages(int page, int rows, int totalRecords)
        {
            // get the index where to start retrieving records from
            // 0 = starts from the beggining, 10 means skip the first 10 records and start from record 11
            int startRowIndex = ((page * rows) - rows);

            // calculate the total number of pages
            int totalPages = (int)Math.Ceiling((float)totalRecords / (float)rows);

            return (startRowIndex, totalPages);
        }

        /// <summary>
        /// Registers Model services.  Used by the Starup class.
        /// </summary>
        public static void AddModelServices(IServiceCollection services)
        {
            services.AddScoped<Category>();
            services.AddScoped<Supplier>();
            services.AddScoped<Product>();
        }

        /// <summary>
        /// Registers View Model services.  Used by the Starup class.
        /// </summary>
        public static void AddViewModelServices(IServiceCollection services)
        {
            services.AddScoped<CategoryViewModel>();
            services.AddScoped<SupplierViewModel>();
            services.AddScoped<ProductViewModel>();

            services.AddScoped<CategoryForeachViewModel>();
            services.AddScoped<SupplierForeachViewModel>();
            services.AddScoped<ProductForeachViewModel>();
        }

        /// <summary>
        /// Registers Data Repository and Business Layer services.  Used by the Starup class.
        /// </summary>
        public static void AddDataRepositoryAndBusinessLayerServices(IServiceCollection services, string connectionString)
        {
            // instantiate repositories
            ICategoryRepository categoryRepository = new CategoryRepository(connectionString);
            ISupplierRepository supplierRepository = new SupplierRepository(connectionString);
            IProductRepository productRepository = new ProductRepository(connectionString);

            // instantiate business layers
            ICategoryBusinessLayer categoryBusinessLayer = new CategoryBusinessLayer(categoryRepository);
            ISupplierBusinessLayer supplierBusinessLayer = new SupplierBusinessLayer(supplierRepository);
            IProductBusinessLayer productBusinessLayer = new ProductBusinessLayer(productRepository, supplierBusinessLayer, categoryBusinessLayer);

            // add data repository scoped services
            services.AddScoped<ICategoryRepository>(e => categoryRepository);
            services.AddScoped<ISupplierRepository>(e => supplierRepository);
            services.AddScoped<IProductRepository>(e => productRepository);

            // add business layer scoped services
            services.AddScoped<ICategoryBusinessLayer>(e => categoryBusinessLayer);
            services.AddScoped<ISupplierBusinessLayer>(e => supplierBusinessLayer);
            services.AddScoped<IProductBusinessLayer>(e => productBusinessLayer);
        }
    }
}
