using Microsoft.AspNetCore.Mvc;
using StoredProcWaApi.BusinessLayer;
using StoredProcWaApi.Models;
using StoredProcWaApi.Domain;
using Newtonsoft.Json;

namespace StoredProcWaSrvcs.ApiControllers.Base
{
     /// <summary>
     /// Base class for CategoryApiController.
     /// ************************************* Do not make changes to this class *************************************
     /// ** Put your additional code in the CategoryApiController class. **
     /// *************************************************************************************************************
     /// </summary>
     [ApiController]
     public partial class CategoryApiController : ControllerBase
     {
         private Category _category;
         private readonly ICategoryBusinessLayer _categoryBusinessLayer;

         // constructor
         public CategoryApiController(Category category, ICategoryBusinessLayer categoryBusinessLayer)
         {
             _category = category;
             _categoryBusinessLayer = categoryBusinessLayer;
         }

         /// <summary>
         /// Inserts/Adds/Creates a new record in the database
         /// </summary>
         /// <param name="model">Pass the Category here.  Arrives as CategoryFields which automatically strips the data annotations from the Category.</param>
         /// <returns>Task of IActionResult</returns>
         [Route("[controller]/insert")]
         [HttpPost]
         public async Task<IActionResult> Insert([FromBody]Category model, bool isForListInlineOrListCrud = false)
         {
             // add a new record
             return await AddEditCategoryAsync(model, CrudOperation.Add, isForListInlineOrListCrud);
         }

         /// <summary>
         /// Updates an existing record in the database by primary key.  Pass the primary key in the Category
         /// </summary>
         /// <param name="model">Pass the Category here.  Arrives as CategoryFields which automatically strips the data annotations from the Category.</param>
         /// <returns>Task of IActionResult</returns>
         [Route("[controller]/update")]
         [HttpPost]
         public async Task<IActionResult> Update([FromBody]Category model, bool isForListInlineOrListCrud = false)
         {
             // update existing record
             return await this.AddEditCategoryAsync(model, CrudOperation.Update, isForListInlineOrListCrud);
         }

         /// <summary>
         /// Deletes an existing record by primary key
         /// </summary>
         /// <param name="id">CategoryID</param>
         /// <returns>Task of IActionResult</returns>
         [Route("[controller]/delete")]
         [HttpDelete]
         public async Task<IActionResult> Delete(int id)
         {
             try
             {
                 // delete a record based on id(s)
                 await _categoryBusinessLayer.DeleteAsync(id);

                 // everthing went well
                 return Ok();
             }
             catch (Exception ex)
             {

                 // something went wrong
                 return BadRequest("Error Message: " + ex.Message);
             }
         }

         /// <summary>
         /// Deletes multiple records based on the comma-delimited ids (primary keys)
         /// </summary>
         /// <param name="ids">List of CategoryIDs</param>
         /// <returns>Task of IActionResult</returns>
         [Route("[controller]/deletemultiple")]
         [HttpDelete]
         public async Task<IActionResult> DeleteMultiple(string ids)
         {
             try
             {
                 // split ids into a List
                 List<Int32> categoryIDList = ids.Split(",").Select(Int32.Parse).ToList();

                 // delete multiple records based on a list of ids (primary keys)
                 await _categoryBusinessLayer.DeleteMultipleAsync(categoryIDList);

                 // everthing went well
                 return Ok();
             }
             catch (Exception ex)
             {
                 // something went wrong
                 return BadRequest("Error Message: " + ex.Message);
             }
         }

         /// <summary>
         /// Selects records as a collection (List) of Categories sorted by the sord, starting in page, get the number of rows.
         /// </summary>
         /// <param name="sidx">Field to sort.  Can be an empty string.</param>
         /// <param name="sord">asc or an empty string = ascending.  desc = descending</param>
         /// <param name="page">Current page</param>
         /// <param name="rows">Number of rows to retrieve</param>
         /// <returns>Returns a collection (List) of Categories</returns>
         [Route("[controller]/selectskipandtake")]
         [HttpGet]
         public async Task<List<Category>> SelectSkipAndTake(string sidx, string sord, int page, int rows)
         {
             // get the index where to start retrieving records from
             // 0 = starts from the beggining, 10 means skip the first 10 records and start from record 11
             int startRowIndex = ((page * rows) - rows);

             // get records
             List<Category> objCategoriesList = await _categoryBusinessLayer.SelectSkipAndTakeAsync(rows, startRowIndex, sidx + " " + sord);
             return objCategoriesList;
         }

         /// <summary>
         /// Selects records as a collection (List) of Categories sorted by the sord, starting in page, get the number of rows.
         /// </summary>
         /// <param name="_search">true or false</param>
         /// <param name="nd">nd</param>
         /// <param name="rows">Number of rows to retrieve</param>
         /// <param name="page">Current page</param>
         /// <param name="sidx">Field to sort.  Can be an empty string.</param>
         /// <param name="sord">asc or an empty string = ascending.  desc = descending</param>
         /// <param name="filters">Optional.  Filters used in search</param>
         /// <returns>Returns a collection (List) of Categories</returns>
         [Route("[controller]/selectskipandtakewithfilters")]
         [HttpGet]
         public async Task<List<Category>> SelectSkipAndTakeWithFilters(string _search, string nd, int rows, int page, string sidx, string sord, string filters = "")
         {
             int? categoryID = null;
             string categoryName = String.Empty;

             if (!String.IsNullOrEmpty(filters))
             {
                 // deserialize filters and get values being searched
                 var jsonResult = JsonConvert.DeserializeObject<Dictionary<string, dynamic>>(filters);

                 foreach (var rule in jsonResult["rules"])
                 {
                     if (rule["field"].Value.ToLower() == "categoryid")
                         categoryID = Convert.ToInt32(rule["data"].Value);

                     if (rule["field"].Value.ToLower() == "categoryname")
                         categoryName = rule["data"].Value;

                 }

                 // sometimes jqgrid assigns a -1 to numeric fields when no value is assigned
                 // instead of assigning a null, we'll correct this here
                 if (categoryID == -1)
                     categoryID = null;

             }

             // get the index where to start retrieving records from
             // 0 = starts from the beggining, 10 means skip the first 10 records and start from record 11
             int startRowIndex = ((page * rows) - rows);

             // get records based on filters
             List<Category> objCategoriesList = await _categoryBusinessLayer.SelectSkipAndTakeDynamicWhereAsync(categoryID, categoryName, rows, startRowIndex, sidx + " " + sord);
             return objCategoriesList;
         }

         /// <summary>
         /// Selects records as a collection (List) of Categories sorted by the sord, starting in page, get the number of rows.
         /// </summary>
         /// <param name="sidx">Field to sort.  Can be an empty string.</param>
         /// <param name="sord">asc or an empty string = ascending.  desc = descending</param>
         /// <param name="page">Current page</param>
         /// <param name="rows">Number of rows to retrieve</param>
         /// <returns>Returns a collection (List) of Categories</returns>
         [Route("[controller]/selectskipandtakewithtotals")]
         [HttpGet]
         public async Task<List<Category>> SelectSkipAndTakeWithTotals(string sidx, string sord, int page, int rows)
         {
             // get the index where to start retrieving records from
             // 0 = starts from the beggining, 10 means skip the first 10 records and start from record 11
             int startRowIndex = ((page * rows) - rows);

             // get records
             List<Category> objCategoriesList = await _categoryBusinessLayer.SelectSkipAndTakeAsync(rows, startRowIndex, sidx + " " + sord);
             return objCategoriesList;
         }

         /// <summary>
         /// Selects a record by primary key(s)
         /// </summary>
         /// <param name="id">CategoryID</param>
         /// <returns>Returns one Category record</returns>
         [Route("[controller]/selectbyprimarykey")]
         [HttpGet]
         public async Task<Category> SelectByPrimaryKey(int id)
         {
             Category objCategory = await _categoryBusinessLayer.SelectByPrimaryKeyAsync(id);
             return objCategory;
         }

         /// <summary>
         /// Gets the total number of records in the Categories table
         /// </summary>
         /// <returns>Returns the Total number of records in the Category table</returns>
         [Route("[controller]/getrecordcount")]
         [HttpGet]
         public async Task<int> GetRecordCount()
         {
             return await _categoryBusinessLayer.GetRecordCountAsync();
         }

         /// <summary>
         /// Gets the total number of records in the Categories table based on search parameters
         /// </summary>
         /// <param name="categoryID">CategoryID</param>
         /// <param name="categoryName">CategoryName</param>
         /// <returns>Returns the Total number of records in the Categories table based on the search parameters</returns>
         [Route("[controller]/getrecordcountdynamicwhere")]
         [HttpGet]
         public async Task<int> GetRecordCountDynamicWhere(int? categoryID, string categoryName)
         {
             return await _categoryBusinessLayer.GetRecordCountDynamicWhereAsync(categoryID, categoryName);
         }

         /// <summary>
         /// Selects records as a collection (List) of Category sorted by the sortByExpression.
         /// </summary>
         /// <param name="rows">Number of rows to retrieve</param>
         /// <param name="startRowIndex">Zero-based.  Row index where to start taking rows from</param>
         /// <param name="sortByExpression">Field to sort and sort direction.  E.g. "FieldName asc" or "FieldName desc"</param>
         /// <returns>Returns Category (List of) collection.</returns>
         [Route("[controller]/selectskipandtakebysortexprsn")]
         [HttpGet]
         public async Task<List<Category>> SelectSkipAndTakeBySortExprsn(int rows, int startRowIndex, string sortByExpression)
         {
             sortByExpression = this.GetSortExpression(sortByExpression);
             return await _categoryBusinessLayer.SelectSkipAndTakeAsync(rows, startRowIndex, sortByExpression);
         }

         /// <summary>
         /// Selects records as a collection (List) of Category sorted by the sortByExpression starting from the startRowIndex, based on the search parameters
         /// </summary>
         /// <param name="categoryID">CategoryID</param>
         /// <param name="categoryName">CategoryName</param>
         /// <param name="rows">Number of rows to retrieve</param>
         /// <param name="startRowIndex">Zero-based.  Row index where to start taking rows from</param>
         /// <param name="sortByExpression">Field to sort and sort direction.  E.g. "FieldName asc" or "FieldName desc"</param>
         /// <param name="page">Page of the grid to show</param>
         /// <returns>Returns a collection (List) of Category</returns>
         [Route("[controller]/selectskipandtakedynamicwhere")]
         [HttpGet]
         public async Task<List<Category>> SelectSkipAndTakeDynamicWhere(int? categoryID, string categoryName, int rows, int startRowIndex, string sortByExpression)
         {
             sortByExpression = this.GetSortExpression(sortByExpression);
             List<Category> objCategoriesList = await _categoryBusinessLayer.SelectSkipAndTakeDynamicWhereAsync(categoryID, categoryName, rows, startRowIndex, sortByExpression);
             return objCategoriesList;
         }

         /// <summary>
         /// Selects all records as a collection (List) of Category sorted by the sort expression.  Data returned is not sorted when sortByExpression is not passed, null, or empty.
         /// </summary>
         /// <param name="sortByExpression">Field to sort and sort direction.  E.g. "FieldName asc" or "FieldName desc".  sortByExpression is not required.</param>
         /// <returns>Returns a collection (List) of Category</returns>
         [Route("[controller]/selectall")]
         [HttpGet]
         public async Task<List<Category>> SelectAll(string sortByExpression = null)
         {
             List<Category> objCategoriesList;

             if (String.IsNullOrEmpty(sortByExpression))
             {
                 objCategoriesList = await _categoryBusinessLayer.SelectAllAsync();
             }
             else
             {
                 sortByExpression = this.GetSortExpression(sortByExpression);
                 objCategoriesList = await _categoryBusinessLayer.SelectAllAsync(sortByExpression);
             }

             return objCategoriesList;
         }

         /// <summary>
         /// Selects records based on the passed filters as a collection (List) of Category.
         /// </summary>
         /// <param name="categoryID">CategoryID</param>
         /// <param name="categoryName">CategoryName</param>
         /// <returns>Returns a collection (List) of Category</returns>
         [Route("[controller]/selectalldynamicwhere")]
         [HttpGet]
         public async Task<List<Category>> SelectAllDynamicWhere(int? categoryID, string categoryName)
         {
             List<Category> objCategoriesList = await _categoryBusinessLayer.SelectAllDynamicWhereAsync(categoryID, categoryName);
             return objCategoriesList;
         }

         /// <summary>
         /// Selects CategoryID and CategoryName columns for use with a DropDownList web control, ComboBox, CheckedBoxList, ListView, ListBox, etc
         /// </summary>
         /// <returns>Returns a collection (List) of Category</returns>
         [Route("[controller]/selectcategorydropdownlistdata")]
         [HttpGet]
         public async Task<List<Category>> SelectCategoryDropDownListData()
         {
             List<Category> objCategoriesList = await _categoryBusinessLayer.SelectCategoryDropDownListDataAsync();
             return objCategoriesList;
         }

         /// <summary>
         /// Gets the default sort expression when no sort expression is provided
         /// </summary>
         private string GetSortExpression(string sortByExpression)
         {
             // when no sort expression is provided, CategoryID is set as the default in ascending order
             // for ascending order, "asc" is not needed, so it is removed
             if (String.IsNullOrEmpty(sortByExpression) || sortByExpression == " asc")
                 sortByExpression = "CategoryID";
             else if (sortByExpression.Contains(" asc"))
                 sortByExpression = sortByExpression.Replace(" asc", "");

             return sortByExpression;
         }

         /// <summary>
         /// Used when adding a new record or updating an existing record
         /// </summary>
         /// <param name="model">Category</param>
         /// <param name="operation">Operation to Add a new record or Update an existing record</param>
         /// <param name="isForListInlineOrListCrud">Used by the razor Views with ListInline or ListCrud when true</param>
         /// <returns>Task of IActionResult</returns>
         private async Task<IActionResult> AddEditCategoryAsync(Category model, CrudOperation operation, bool isForListInlineOrListCrud = false)
         {
             try
             {
                 // create a new instance of the Category when adding a new record
                 // or, retrieve the record that needs to be updated
                 if (operation == CrudOperation.Add)
                    _category = new();
                 else
                    _category = await _categoryBusinessLayer.SelectByPrimaryKeyAsync(model.CategoryID);

                 // assign values to the Category instance
                 _category.CategoryID = model.CategoryID;
                 _category.CategoryName = model.CategoryName;
                 _category.Description = model.Description;

                 // save the new record, or the updated values of the current record
                 if (operation == CrudOperation.Add)
                    await _categoryBusinessLayer.InsertAsync(_category);
                 else
                    await _categoryBusinessLayer.UpdateAsync(_category);

                 // everthing went well
                 return Ok();
             }
             catch (Exception ex)
             {
                 // something went wrong
                 return BadRequest("Error Message: " + ex.Message);
             }
         }
     }
}
