using Microsoft.AspNetCore.Mvc;
using StoredProcWaApi.BusinessLayer;
using StoredProcWaApi.Models;
using StoredProcWaApi.Domain;
using Newtonsoft.Json;

namespace StoredProcWaSrvcs.ApiControllers.Base
{
     /// <summary>
     /// Base class for SupplierApiController.
     /// ************************************* Do not make changes to this class *************************************
     /// ** Put your additional code in the SupplierApiController class. **
     /// *************************************************************************************************************
     /// </summary>
     [ApiController]
     public partial class SupplierApiController : ControllerBase
     {
         private Supplier _supplier;
         private readonly ISupplierBusinessLayer _supplierBusinessLayer;

         // constructor
         public SupplierApiController(Supplier supplier, ISupplierBusinessLayer supplierBusinessLayer)
         {
             _supplier = supplier;
             _supplierBusinessLayer = supplierBusinessLayer;
         }

         /// <summary>
         /// Inserts/Adds/Creates a new record in the database
         /// </summary>
         /// <param name="model">Pass the Supplier here.  Arrives as SupplierFields which automatically strips the data annotations from the Supplier.</param>
         /// <returns>Task of IActionResult</returns>
         [Route("[controller]/insert")]
         [HttpPost]
         public async Task<IActionResult> Insert([FromBody]Supplier model, bool isForListInlineOrListCrud = false)
         {
             // add a new record
             return await AddEditSupplierAsync(model, CrudOperation.Add, isForListInlineOrListCrud);
         }

         /// <summary>
         /// Updates an existing record in the database by primary key.  Pass the primary key in the Supplier
         /// </summary>
         /// <param name="model">Pass the Supplier here.  Arrives as SupplierFields which automatically strips the data annotations from the Supplier.</param>
         /// <returns>Task of IActionResult</returns>
         [Route("[controller]/update")]
         [HttpPost]
         public async Task<IActionResult> Update([FromBody]Supplier model, bool isForListInlineOrListCrud = false)
         {
             // update existing record
             return await this.AddEditSupplierAsync(model, CrudOperation.Update, isForListInlineOrListCrud);
         }

         /// <summary>
         /// Deletes an existing record by primary key
         /// </summary>
         /// <param name="id">SupplierID</param>
         /// <returns>Task of IActionResult</returns>
         [Route("[controller]/delete")]
         [HttpDelete]
         public async Task<IActionResult> Delete(int id)
         {
             try
             {
                 // delete a record based on id(s)
                 await _supplierBusinessLayer.DeleteAsync(id);

                 // everthing went well
                 return Ok();
             }
             catch (Exception ex)
             {

                 // something went wrong
                 return BadRequest("Error Message: " + ex.Message);
             }
         }

         /// <summary>
         /// Deletes multiple records based on the comma-delimited ids (primary keys)
         /// </summary>
         /// <param name="ids">List of SupplierIDs</param>
         /// <returns>Task of IActionResult</returns>
         [Route("[controller]/deletemultiple")]
         [HttpDelete]
         public async Task<IActionResult> DeleteMultiple(string ids)
         {
             try
             {
                 // split ids into a List
                 List<Int32> supplierIDList = ids.Split(",").Select(Int32.Parse).ToList();

                 // delete multiple records based on a list of ids (primary keys)
                 await _supplierBusinessLayer.DeleteMultipleAsync(supplierIDList);

                 // everthing went well
                 return Ok();
             }
             catch (Exception ex)
             {
                 // something went wrong
                 return BadRequest("Error Message: " + ex.Message);
             }
         }

         /// <summary>
         /// Selects records as a collection (List) of Suppliers sorted by the sord, starting in page, get the number of rows.
         /// </summary>
         /// <param name="sidx">Field to sort.  Can be an empty string.</param>
         /// <param name="sord">asc or an empty string = ascending.  desc = descending</param>
         /// <param name="page">Current page</param>
         /// <param name="rows">Number of rows to retrieve</param>
         /// <returns>Returns a collection (List) of Suppliers</returns>
         [Route("[controller]/selectskipandtake")]
         [HttpGet]
         public async Task<List<Supplier>> SelectSkipAndTake(string sidx, string sord, int page, int rows)
         {
             // get the index where to start retrieving records from
             // 0 = starts from the beggining, 10 means skip the first 10 records and start from record 11
             int startRowIndex = ((page * rows) - rows);

             // get records
             List<Supplier> objSuppliersList = await _supplierBusinessLayer.SelectSkipAndTakeAsync(rows, startRowIndex, sidx + " " + sord);
             return objSuppliersList;
         }

         /// <summary>
         /// Selects records as a collection (List) of Suppliers sorted by the sord, starting in page, get the number of rows.
         /// </summary>
         /// <param name="_search">true or false</param>
         /// <param name="nd">nd</param>
         /// <param name="rows">Number of rows to retrieve</param>
         /// <param name="page">Current page</param>
         /// <param name="sidx">Field to sort.  Can be an empty string.</param>
         /// <param name="sord">asc or an empty string = ascending.  desc = descending</param>
         /// <param name="filters">Optional.  Filters used in search</param>
         /// <returns>Returns a collection (List) of Suppliers</returns>
         [Route("[controller]/selectskipandtakewithfilters")]
         [HttpGet]
         public async Task<List<Supplier>> SelectSkipAndTakeWithFilters(string _search, string nd, int rows, int page, string sidx, string sord, string filters = "")
         {
             int? supplierID = null;
             string companyName = String.Empty;
             string contactName = String.Empty;
             string contactTitle = String.Empty;
             string address = String.Empty;
             string city = String.Empty;
             string region1 = String.Empty;
             string postalCode = String.Empty;
             string country = String.Empty;
             string phone = String.Empty;
             string fax = String.Empty;

             if (!String.IsNullOrEmpty(filters))
             {
                 // deserialize filters and get values being searched
                 var jsonResult = JsonConvert.DeserializeObject<Dictionary<string, dynamic>>(filters);

                 foreach (var rule in jsonResult["rules"])
                 {
                     if (rule["field"].Value.ToLower() == "supplierid")
                         supplierID = Convert.ToInt32(rule["data"].Value);

                     if (rule["field"].Value.ToLower() == "companyname")
                         companyName = rule["data"].Value;

                     if (rule["field"].Value.ToLower() == "contactname")
                         contactName = rule["data"].Value;

                     if (rule["field"].Value.ToLower() == "contacttitle")
                         contactTitle = rule["data"].Value;

                     if (rule["field"].Value.ToLower() == "address")
                         address = rule["data"].Value;

                     if (rule["field"].Value.ToLower() == "city")
                         city = rule["data"].Value;

                     if (rule["field"].Value.ToLower() == "region1")
                         region1 = rule["data"].Value;

                     if (rule["field"].Value.ToLower() == "postalcode")
                         postalCode = rule["data"].Value;

                     if (rule["field"].Value.ToLower() == "country")
                         country = rule["data"].Value;

                     if (rule["field"].Value.ToLower() == "phone")
                         phone = rule["data"].Value;

                     if (rule["field"].Value.ToLower() == "fax")
                         fax = rule["data"].Value;

                 }

                 // sometimes jqgrid assigns a -1 to numeric fields when no value is assigned
                 // instead of assigning a null, we'll correct this here
                 if (supplierID == -1)
                     supplierID = null;

             }

             // get the index where to start retrieving records from
             // 0 = starts from the beggining, 10 means skip the first 10 records and start from record 11
             int startRowIndex = ((page * rows) - rows);

             // get records based on filters
             List<Supplier> objSuppliersList = await _supplierBusinessLayer.SelectSkipAndTakeDynamicWhereAsync(supplierID, companyName, contactName, contactTitle, address, city, region1, postalCode, country, phone, fax, rows, startRowIndex, sidx + " " + sord);
             return objSuppliersList;
         }

         /// <summary>
         /// Selects records as a collection (List) of Suppliers sorted by the sord, starting in page, get the number of rows.
         /// </summary>
         /// <param name="sidx">Field to sort.  Can be an empty string.</param>
         /// <param name="sord">asc or an empty string = ascending.  desc = descending</param>
         /// <param name="page">Current page</param>
         /// <param name="rows">Number of rows to retrieve</param>
         /// <returns>Returns a collection (List) of Suppliers</returns>
         [Route("[controller]/selectskipandtakewithtotals")]
         [HttpGet]
         public async Task<List<Supplier>> SelectSkipAndTakeWithTotals(string sidx, string sord, int page, int rows)
         {
             // get the index where to start retrieving records from
             // 0 = starts from the beggining, 10 means skip the first 10 records and start from record 11
             int startRowIndex = ((page * rows) - rows);

             // get records
             List<Supplier> objSuppliersList = await _supplierBusinessLayer.SelectSkipAndTakeAsync(rows, startRowIndex, sidx + " " + sord);
             return objSuppliersList;
         }

         /// <summary>
         /// Selects a record by primary key(s)
         /// </summary>
         /// <param name="id">SupplierID</param>
         /// <returns>Returns one Supplier record</returns>
         [Route("[controller]/selectbyprimarykey")]
         [HttpGet]
         public async Task<Supplier> SelectByPrimaryKey(int id)
         {
             Supplier objSupplier = await _supplierBusinessLayer.SelectByPrimaryKeyAsync(id);
             return objSupplier;
         }

         /// <summary>
         /// Gets the total number of records in the Suppliers table
         /// </summary>
         /// <returns>Returns the Total number of records in the Supplier table</returns>
         [Route("[controller]/getrecordcount")]
         [HttpGet]
         public async Task<int> GetRecordCount()
         {
             return await _supplierBusinessLayer.GetRecordCountAsync();
         }

         /// <summary>
         /// Gets the total number of records in the Suppliers table based on search parameters
         /// </summary>
         /// <param name="supplierID">SupplierID</param>
         /// <param name="companyName">CompanyName</param>
         /// <param name="contactName">ContactName</param>
         /// <param name="contactTitle">ContactTitle</param>
         /// <param name="address">Address</param>
         /// <param name="city">City</param>
         /// <param name="region1">Region1</param>
         /// <param name="postalCode">PostalCode</param>
         /// <param name="country">Country</param>
         /// <param name="phone">Phone</param>
         /// <param name="fax">Fax</param>
         /// <returns>Returns the Total number of records in the Suppliers table based on the search parameters</returns>
         [Route("[controller]/getrecordcountdynamicwhere")]
         [HttpGet]
         public async Task<int> GetRecordCountDynamicWhere(int? supplierID, string companyName, string contactName, string contactTitle, string address, string city, string region1, string postalCode, string country, string phone, string fax)
         {
             return await _supplierBusinessLayer.GetRecordCountDynamicWhereAsync(supplierID, companyName, contactName, contactTitle, address, city, region1, postalCode, country, phone, fax);
         }

         /// <summary>
         /// Selects records as a collection (List) of Supplier sorted by the sortByExpression.
         /// </summary>
         /// <param name="rows">Number of rows to retrieve</param>
         /// <param name="startRowIndex">Zero-based.  Row index where to start taking rows from</param>
         /// <param name="sortByExpression">Field to sort and sort direction.  E.g. "FieldName asc" or "FieldName desc"</param>
         /// <returns>Returns Supplier (List of) collection.</returns>
         [Route("[controller]/selectskipandtakebysortexprsn")]
         [HttpGet]
         public async Task<List<Supplier>> SelectSkipAndTakeBySortExprsn(int rows, int startRowIndex, string sortByExpression)
         {
             sortByExpression = this.GetSortExpression(sortByExpression);
             return await _supplierBusinessLayer.SelectSkipAndTakeAsync(rows, startRowIndex, sortByExpression);
         }

         /// <summary>
         /// Selects records as a collection (List) of Supplier sorted by the sortByExpression starting from the startRowIndex, based on the search parameters
         /// </summary>
         /// <param name="supplierID">SupplierID</param>
         /// <param name="companyName">CompanyName</param>
         /// <param name="contactName">ContactName</param>
         /// <param name="contactTitle">ContactTitle</param>
         /// <param name="address">Address</param>
         /// <param name="city">City</param>
         /// <param name="region1">Region1</param>
         /// <param name="postalCode">PostalCode</param>
         /// <param name="country">Country</param>
         /// <param name="phone">Phone</param>
         /// <param name="fax">Fax</param>
         /// <param name="rows">Number of rows to retrieve</param>
         /// <param name="startRowIndex">Zero-based.  Row index where to start taking rows from</param>
         /// <param name="sortByExpression">Field to sort and sort direction.  E.g. "FieldName asc" or "FieldName desc"</param>
         /// <param name="page">Page of the grid to show</param>
         /// <returns>Returns a collection (List) of Supplier</returns>
         [Route("[controller]/selectskipandtakedynamicwhere")]
         [HttpGet]
         public async Task<List<Supplier>> SelectSkipAndTakeDynamicWhere(int? supplierID, string companyName, string contactName, string contactTitle, string address, string city, string region1, string postalCode, string country, string phone, string fax, int rows, int startRowIndex, string sortByExpression)
         {
             sortByExpression = this.GetSortExpression(sortByExpression);
             List<Supplier> objSuppliersList = await _supplierBusinessLayer.SelectSkipAndTakeDynamicWhereAsync(supplierID, companyName, contactName, contactTitle, address, city, region1, postalCode, country, phone, fax, rows, startRowIndex, sortByExpression);
             return objSuppliersList;
         }

         /// <summary>
         /// Selects all records as a collection (List) of Supplier sorted by the sort expression.  Data returned is not sorted when sortByExpression is not passed, null, or empty.
         /// </summary>
         /// <param name="sortByExpression">Field to sort and sort direction.  E.g. "FieldName asc" or "FieldName desc".  sortByExpression is not required.</param>
         /// <returns>Returns a collection (List) of Supplier</returns>
         [Route("[controller]/selectall")]
         [HttpGet]
         public async Task<List<Supplier>> SelectAll(string sortByExpression = null)
         {
             List<Supplier> objSuppliersList;

             if (String.IsNullOrEmpty(sortByExpression))
             {
                 objSuppliersList = await _supplierBusinessLayer.SelectAllAsync();
             }
             else
             {
                 sortByExpression = this.GetSortExpression(sortByExpression);
                 objSuppliersList = await _supplierBusinessLayer.SelectAllAsync(sortByExpression);
             }

             return objSuppliersList;
         }

         /// <summary>
         /// Selects records based on the passed filters as a collection (List) of Supplier.
         /// </summary>
         /// <param name="supplierID">SupplierID</param>
         /// <param name="companyName">CompanyName</param>
         /// <param name="contactName">ContactName</param>
         /// <param name="contactTitle">ContactTitle</param>
         /// <param name="address">Address</param>
         /// <param name="city">City</param>
         /// <param name="region1">Region1</param>
         /// <param name="postalCode">PostalCode</param>
         /// <param name="country">Country</param>
         /// <param name="phone">Phone</param>
         /// <param name="fax">Fax</param>
         /// <returns>Returns a collection (List) of Supplier</returns>
         [Route("[controller]/selectalldynamicwhere")]
         [HttpGet]
         public async Task<List<Supplier>> SelectAllDynamicWhere(int? supplierID, string companyName, string contactName, string contactTitle, string address, string city, string region1, string postalCode, string country, string phone, string fax)
         {
             List<Supplier> objSuppliersList = await _supplierBusinessLayer.SelectAllDynamicWhereAsync(supplierID, companyName, contactName, contactTitle, address, city, region1, postalCode, country, phone, fax);
             return objSuppliersList;
         }

         /// <summary>
         /// Selects SupplierID and CompanyName columns for use with a DropDownList web control, ComboBox, CheckedBoxList, ListView, ListBox, etc
         /// </summary>
         /// <returns>Returns a collection (List) of Supplier</returns>
         [Route("[controller]/selectsupplierdropdownlistdata")]
         [HttpGet]
         public async Task<List<Supplier>> SelectSupplierDropDownListData()
         {
             List<Supplier> objSuppliersList = await _supplierBusinessLayer.SelectSupplierDropDownListDataAsync();
             return objSuppliersList;
         }

         /// <summary>
         /// Gets the default sort expression when no sort expression is provided
         /// </summary>
         private string GetSortExpression(string sortByExpression)
         {
             // when no sort expression is provided, SupplierID is set as the default in ascending order
             // for ascending order, "asc" is not needed, so it is removed
             if (String.IsNullOrEmpty(sortByExpression) || sortByExpression == " asc")
                 sortByExpression = "SupplierID";
             else if (sortByExpression.Contains(" asc"))
                 sortByExpression = sortByExpression.Replace(" asc", "");

             return sortByExpression;
         }

         /// <summary>
         /// Used when adding a new record or updating an existing record
         /// </summary>
         /// <param name="model">Supplier</param>
         /// <param name="operation">Operation to Add a new record or Update an existing record</param>
         /// <param name="isForListInlineOrListCrud">Used by the razor Views with ListInline or ListCrud when true</param>
         /// <returns>Task of IActionResult</returns>
         private async Task<IActionResult> AddEditSupplierAsync(Supplier model, CrudOperation operation, bool isForListInlineOrListCrud = false)
         {
             try
             {
                 // create a new instance of the Supplier when adding a new record
                 // or, retrieve the record that needs to be updated
                 if (operation == CrudOperation.Add)
                    _supplier = new();
                 else
                    _supplier = await _supplierBusinessLayer.SelectByPrimaryKeyAsync(model.SupplierID);

                 // assign values to the Supplier instance
                 _supplier.SupplierID = model.SupplierID;
                 _supplier.CompanyName = model.CompanyName;
                 _supplier.ContactName = model.ContactName;
                 _supplier.ContactTitle = model.ContactTitle;
                 _supplier.Address = model.Address;
                 _supplier.City = model.City;
                 _supplier.Region1 = model.Region1;
                 _supplier.PostalCode = model.PostalCode;
                 _supplier.Country = model.Country;
                 _supplier.Phone = model.Phone;
                 _supplier.Fax = model.Fax;
                 _supplier.HomePage = model.HomePage;

                 // save the new record, or the updated values of the current record
                 if (operation == CrudOperation.Add)
                    await _supplierBusinessLayer.InsertAsync(_supplier);
                 else
                    await _supplierBusinessLayer.UpdateAsync(_supplier);

                 // everthing went well
                 return Ok();
             }
             catch (Exception ex)
             {
                 // something went wrong
                 return BadRequest("Error Message: " + ex.Message);
             }
         }
     }
}
